﻿using C1.WPF.FlexGrid;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WPF_Flex_NestedGrids
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static DataSet obj_DataSet;

        public MainWindow()
        {
            InitializeComponent();

            obj_DataSet = new DataSet();
            obj_DataSet.Tables.Add(MyDataClass.CreateCategoryTable());
            obj_DataSet.Tables.Add(MyDataClass.CreateProductTable());

            c1Flexgrid1.ItemsSource = obj_DataSet.Tables["Category"].DefaultView;
            c1Flexgrid1.AllowMerging = AllowMerging.Cells;

            c1Flexgrid1.CellFactory = new MyCellFactory();

            #region Include Unbound Child Rows to hold the Nested Grids
            for (int i = c1Flexgrid1.Rows.Count - 1; i >= 0; i = i - 1)
            {
                C1.WPF.FlexGrid.Row rw = new C1.WPF.FlexGrid.Row();
                rw.AllowMerging = true;
                c1Flexgrid1.Rows.Insert(i + 1, rw);
                rw.Height = 1;
            }
            #endregion

            // Merge cells for Child Grids
            c1Flexgrid1.MergeManager = new MyMergeManager();
        }

    }

    /// <summary>
    /// DataSource: Linked DataTables
    /// </summary>
    public class MyDataClass
    {

        public static DataTable CreateCategoryTable()
        {
            System.Data.DataTable obj_DataTable = new System.Data.DataTable("Category");
            //Declaring the array of DataColum to hold the Primary Key Columns
            System.Data.DataColumn[] obj_PrimaryClmn = new System.Data.DataColumn[2];
            System.Data.DataRow obj_DataRow = null;

            obj_DataTable.Columns.Add(new System.Data.DataColumn("CategoryId"));
            obj_DataTable.Columns.Add(new System.Data.DataColumn("CategoryName"));

            obj_PrimaryClmn[0] = obj_DataTable.Columns[0];
            //Assigning the CategoryId column as Primary Key
            obj_DataTable.PrimaryKey = obj_PrimaryClmn;


            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 1;
            obj_DataRow[1] = "Computers";
            obj_DataTable.Rows.Add(obj_DataRow);

            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 2;
            obj_DataRow[1] = "Drinks";
            obj_DataTable.Rows.Add(obj_DataRow);

            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 3;
            obj_DataRow[1] = "Snacks";
            obj_DataTable.Rows.Add(obj_DataRow);

            return obj_DataTable;

        }

        public static DataTable CreateProductTable()
        {
            System.Data.DataRow obj_DataRow = null;
            System.Data.DataTable obj_DataTable = new System.Data.DataTable("Product");

            obj_DataTable.Columns.Add(new System.Data.DataColumn("ProductId"));
            obj_DataTable.Columns.Add(new System.Data.DataColumn("ProductName"));
            obj_DataTable.Columns.Add(new System.Data.DataColumn("CategoryId"));

            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 1;
            obj_DataRow[1] = "PHP";
            obj_DataRow[2] = 1;
            obj_DataTable.Rows.Add(obj_DataRow);

            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 2;
            obj_DataRow[1] = "Dew";
            obj_DataRow[2] = 2;
            obj_DataTable.Rows.Add(obj_DataRow);

            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 3;
            obj_DataRow[1] = "ASP.Net";
            obj_DataRow[2] = 1;
            obj_DataTable.Rows.Add(obj_DataRow);

            obj_DataRow = obj_DataTable.NewRow();
            obj_DataRow[0] = 4;
            obj_DataRow[1] = "Pizza";
            obj_DataRow[2] = 3;
            obj_DataTable.Rows.Add(obj_DataRow);

            return obj_DataTable;
        }
    }

    /// <summary>
    /// Merging Cells for Child Unbound Row
    /// </summary>
    public class MyMergeManager : C1.WPF.FlexGrid.IMergeManager
    {
        public CellRange GetMergedRange(C1FlexGrid grid, CellType cellType, CellRange range)
        {
            if (grid.Rows[range.Row].DataItem == null)
                if (cellType == CellType.Cell && range.Column >= 1 && range.Column < grid.Columns.Count-1)
                {
                    range.Column = 1;
                    range.Column2 = grid.Columns.Count-1;
                }
            return range;
        }
    }

    public class MyCellFactory : C1.WPF.FlexGrid.CellFactory
    {
        Row _gr;
        DataSet tempDataSet;
        C1.WPF.FlexGrid.C1FlexGrid _fg;
        static ImageSource _bmpExpanded, _bmpCollapsed;
        public static List<int> expandedList = new List<int>();

        public MyCellFactory()
        {
            //Read Expand/Collapse icons from Resource folder
            _bmpExpanded = ImageCell.GetImageSource("Expanded.png");
            _bmpCollapsed = ImageCell.GetImageSource("Collapsed.png");
        }

        public override void CreateCellContent(C1FlexGrid grid, Border bdr, CellRange rng)
        {
            base.CreateCellContent(grid, bdr, rng);
            if (_fg == null)
                _fg = grid;

            tempDataSet = (grid.ItemsSource as DataView).Table.DataSet.Copy();

            #region Code Block to include Expand/Collapse icon for Bound Row if Child Row exists
            if (_fg.Rows[rng.Row].DataItem != null)
            {
                DataRowView drv = (grid.Rows[rng.Row].DataItem as System.Data.DataRowView);
                tempDataSet.Tables["Product"].DefaultView.RowFilter = "CategoryID=" + drv.Row["CategoryID"];

                if (rng.Column == 0)
                {

                    if (tempDataSet.Tables["Product"].DefaultView.Count > 0)
                    {
                        bdr.Child = null;
                        Image _nodeImage;
                        _gr = grid.Rows[rng.Row];
                        _nodeImage = new Image();

                        if (MyCellFactory.expandedList.Contains(rng.Row))
                            _nodeImage.Source = _bmpExpanded;
                        else
                            _nodeImage.Source = _bmpCollapsed;

                        _nodeImage.Width = _nodeImage.Height = 9;
                        _nodeImage.VerticalAlignment = VerticalAlignment.Center;
                        _nodeImage.Stretch = Stretch.None;
                        bdr.Child = _nodeImage;
                        _nodeImage.PreviewMouseDown += _nodeImage_PreviewMouseDown;
                    }
                    else
                    {
                        expandedList.Remove(rng.Row);
                        _fg.Rows[rng.Row + 1].Height = 1;
                    }
                }

            }
            #endregion
            #region Code Block to include Child C1Flexgrid in the Unbound Row
            else
                if (rng.Column == 1)
                {
                    DataRowView drv = (grid.Rows[rng.Row - 1].DataItem as System.Data.DataRowView);
                    tempDataSet.Tables["Product"].DefaultView.RowFilter = "CategoryID=" + drv.Row["CategoryID"];

                    C1.WPF.FlexGrid.C1FlexGrid cfl = new C1FlexGrid();
                    cfl.HeadersVisibility = HeadersVisibility.Column;
                    cfl.ItemsSource = tempDataSet.Tables["Product"].DefaultView;
                    cfl.VerticalAlignment = VerticalAlignment.Top;

                    cfl.Height = ((cfl.Rows.Count + 2) * cfl.Rows.DefaultSize);
                    if (grid.Rows[rng.Row].ActualHeight < cfl.Height || grid.Rows[rng.Row].ActualHeight > cfl.Height)
                    {
                        grid.Rows[rng.Row].Tag = cfl.Height;                        
                    }                   

                    bdr.Child = cfl;
                }
            #endregion

        }

        #region Handle Mouse Down event for toggling the expand/Collapse icon and change the visibility of Child Row
        void _nodeImage_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            int _row = Grid.GetRow(((VisualTreeHelper.GetParent(e.OriginalSource as Image) as Border)));
            SetExpandCollapse(_row);
        }

        public void SetExpandCollapse(int _row)
        {
            if (expandedList.Contains(_row))
            {
                _fg.Rows[_row + 1].Height = 1;
                expandedList.Remove(_row);
            }
            else
            {
                _fg.Rows[_row + 1].Height = int.Parse(_fg.Rows[_row + 1].Tag.ToString());
                expandedList.Add(_row);
            }
        }

        #endregion
    }

    /// <summary>
    /// Returns Image for Expanded and Collpased icon
    /// </summary>
    public abstract class ImageCell : StackPanel
    {
        ImageSource _imgSrc;

        public ImageCell(Row row)
        {
            if (_imgSrc == null)
            {
                _imgSrc = GetImageSource(GetImageResourceName());
            }

            Orientation = Orientation.Horizontal;

            var img = new Image();
            img.Source = _imgSrc;
            img.Width = 25;
            img.Height = 15;
            img.VerticalAlignment = VerticalAlignment.Center;
            img.Stretch = Stretch.None;
            Children.Add(img);
        }

        public abstract string GetImageResourceName();

        public static ImageSource GetImageSource(string imageName)
        {
            var imgConv = new ImageSourceConverter();
            string path = string.Format("pack://application:,,,/WPF_Flex_NestedGrids;component/Resources/{0}", imageName);
            return (ImageSource)imgConv.ConvertFromString(path);
        }
    }

}
